@@ -12,7 +12,7 @@ use warnings;
 
 package Dist::Zilla::Plugin::Git::CommitBuild;
 {
-  $Dist::Zilla::Plugin::Git::CommitBuild::VERSION = '1.121010';
+  $Dist::Zilla::Plugin::Git::CommitBuild::VERSION = '1.121750';
 }
 # ABSTRACT: checkin build results on separate branch
 
@@ -22,9 +22,15 @@ use File::chdir;
 use File::Spec::Functions qw/ rel2abs catfile /;
 use File::Temp;
 use Moose;
+use namespace::autoclean;
+use MooseX::AttributeShortcuts;
+use Path::Class;
+use MooseX::Types::Path::Class ':all';
 use MooseX::Has::Sugar;
 use MooseX::Types::Moose qw{ Str };
 use Cwd qw(abs_path);
+use Try::Tiny;
+
 use String::Formatter (
 	method_stringf => {
 		-as   => '_format_branch',
@@ -44,6 +50,9 @@ use String::Formatter (
 	}
 );
 
+# debugging...
+#use Smart::Comments '###';
+
 with 'Dist::Zilla::Role::AfterBuild', 'Dist::Zilla::Role::AfterRelease';
 with 'Dist::Zilla::Role::Git::Repo';
 
@@ -53,7 +62,7 @@ has branch  => ( ro, isa => Str, default => 'build/%b', required => 1 );
 has release_branch  => ( ro, isa => Str, required => 0 );
 has message => ( ro, isa => Str, default => 'Build results of %h (on %b)', required => 1 );
 has release_message => ( ro, isa => Str, lazy => 1, builder => '_build_release_message' );
-has build_root => ( rw );
+has build_root => ( rw, coerce => 1, isa => Dir );
 has _git => (rw, weak_ref => 1);
 
 # -- attribute builders
@@ -87,34 +96,27 @@ sub _commit_build {
     my $src     = Git::Wrapper->new( $self->repo_root );
     $self->_git($src);
 
-    my $dir = rel2abs( $self->build_root );
-
-    my $tree = do {
-        # don't overwrite the user's index
-        local $ENV{GIT_INDEX_FILE} = catfile( $tmp_dir, "temp_git_index" );
-        local $ENV{GIT_DIR}        = catfile( $CWD,     '.git' );
-        local $ENV{GIT_WORK_TREE}  = $dir;
-
-        local $CWD = $dir;
-
-        my $write_tree_repo = Git::Wrapper->new( $self->repo_root );
-
-        $write_tree_repo->add({ v => 1, force => 1}, '.' );
-        ($write_tree_repo->write_tree)[0];
-    };
-
     my $target_branch = _format_branch( $branch, $src );
+    my $dir           = $self->build_root;
 
-    # no change, abort
-    return
-      if eval {
-              $src->rev_parse({q=>1, verify => 1}, $target_branch );
-        }
-          and not $src->diff({ 'stat' => 1 }, $target_branch, $tree );
+    # returns the sha1 of the created tree object
+    my $tree = $self->_create_tree($src, $dir);
+
+    my ($last_build_tree) = try { $src->rev_parse("$target_branch^{tree}") };
+    $last_build_tree ||= 'none';
+
+    ### $last_build_tree
+    if ($tree eq $last_build_tree) {
+
+        $self->log("No changes since the last build; not committing");
+        return;
+    }
 
     my @parents = grep {
         eval { $src->rev_parse({ 'q' => 1, 'verify'=>1}, $_ ) }
-    } $target_branch, 'HEAD';
+    } $target_branch;
+
+    ### @parents
 
     my @commit;
     {
@@ -135,9 +137,51 @@ sub _commit_build {
 
     }
 
+    ### @commit
     $src->update_ref( 'refs/heads/' . $target_branch, $commit[0] );
 }
 
+sub _create_tree {
+    my ($self, $repo, $fs_obj) = @_;
+
+    ### called with: "$fs_obj"
+    if (!$fs_obj->is_dir) {
+
+        my ($sha) = $repo->hash_object({ w => 1 }, "$fs_obj");
+        ### hashed: "$sha $fs_obj"
+        return $sha;
+    }
+
+    my @entries;
+    for my $obj ($fs_obj->children) {
+
+        ### working on: "$obj"
+        my $sha  = $self->_create_tree($repo, $obj);
+        my $mode = sprintf('%o', $obj->stat->mode); # $obj->is_dir ? '040000' : '
+        my $type = $obj->is_dir ? 'tree' : 'blob';
+        my $name = $obj->basename;
+
+        push @entries, "$mode $type $sha\t$name";
+    }
+
+    ### @entries
+
+    my $sha = do {
+        use autodie 'system';
+        my $cmd = join '\n', @entries, q{};
+        $cmd  = qq{echo -en "$cmd" | git mktree};
+
+        #### $cmd
+        my $sha = `$cmd`;
+        chomp $sha;
+
+        ### tree created at: $sha
+        $sha;
+    };
+
+    return $sha;
+}
+
 1;
 
 
@@ -149,7 +193,7 @@ Dist::Zilla::Plugin::Git::CommitBuild - checkin build results on separate branch
 
 =head1 VERSION
 
-version 1.121010
+version 1.121750
 
 =head1 SYNOPSIS
 
